use log::warn;
use rocket::{
    delete, get,
    http::CookieJar,
    patch, post,
    serde::json::{json, Json},
    State,
};
use serde::{Deserialize, Serialize};
use serenity::prelude::Context;

use crate::web::{
    check_authorization, guards::transaction::Transaction, routes::JsonResult, string_opt,
};

#[derive(Deserialize)]
pub struct CreateTodo {
    channel_id: Option<u64>,
    value: String,
}

#[derive(Serialize)]
struct GetTodo {
    id: u32,
    #[serde(with = "string_opt")]
    channel_id: Option<u64>,
    value: String,
}

#[post("/api/guild/<id>/todos", data = "<todo>")]
pub async fn create_todo(
    id: u64,
    todo: Json<CreateTodo>,
    cookies: &CookieJar<'_>,
    ctx: &State<Context>,
    mut transaction: Transaction<'_>,
) -> JsonResult {
    check_authorization(cookies, ctx.inner(), id).await?;

    Ok(json!({}))
}

#[get("/api/guild/<id>/todos")]
pub async fn get_todo(
    id: u64,
    cookies: &CookieJar<'_>,
    ctx: &State<Context>,
    mut transaction: Transaction<'_>,
) -> JsonResult {
    check_authorization(cookies, ctx.inner(), id).await?;

    let todos = sqlx::query_as!(
        GetTodo,
        "
        SELECT
            todos.id,
            channels.channel AS channel_id,
            value
        FROM todos
        INNER JOIN guilds
        ON guilds.id = todos.guild_id
        LEFT JOIN channels
        ON channels.id = todos.channel_id
        WHERE guilds.guild = ?
        ",
        id
    )
    .fetch_all(transaction.executor())
    .await
    .map_err(|e| {
        warn!("Error fetching todos: {:?}", e);
        json!({ "errors": vec!["Unknown error"] })
    })?;

    Ok(json!(todos))
}

#[patch("/api/guild/<id>/todos")]
pub async fn update_todo(
    id: u64,
    cookies: &CookieJar<'_>,
    ctx: &State<Context>,
    mut transaction: Transaction<'_>,
) -> JsonResult {
    check_authorization(cookies, ctx.inner(), id).await?;

    Ok(json!({}))
}

#[delete("/api/guild/<id>/todos")]
pub async fn delete_todo(
    id: u64,
    cookies: &CookieJar<'_>,
    ctx: &State<Context>,
    mut transaction: Transaction<'_>,
) -> JsonResult {
    check_authorization(cookies, ctx.inner(), id).await?;

    Ok(json!({}))
}
